/*
 * measureDb.cpp
 *
 *  Created on: May 30, 2012
 *      Author: mschoepf
 */

#include "measureDb.h"


measureDbMainWindow::measureDbMainWindow(QWidget* parent) : QMainWindow (parent)
{
	xmem = new xcfMemoryListener();
	maxDb = averageDb = 0.0;
	baseFrame = new QFrame(this);
	baseFrame->setGeometry (0, 0, 1000, 650);
	eqVBoxLayout = new QVBoxLayout(baseFrame);
	ew = new EqualizerWidget();
	eqVBoxLayout->addWidget(ew);
	dbLabel = new QLabel();
	dbLabel->setText(QString ("Level:  0.0 db, LTAverage:  0.0 db, STAverage:  0.0 db, Max:  0.0 db Phase: sleeping"));
	eqVBoxLayout->addWidget(dbLabel);
	eqHBoxLayout = new QHBoxLayout();
	isrStartLabel = new QLabel (QString ("ISR START DB: "));
	eqHBoxLayout->addWidget(isrStartLabel);
	isrStartSpinBox = new QSpinBox ();
	isrStartSpinBox->setRange(1,75);
	isrStartSpinBox->setValue(30);
	eqHBoxLayout->addWidget(isrStartSpinBox);
	isrUttLabel = new QLabel (QString ("ISR UTT DB: "));
	eqHBoxLayout->addWidget(isrUttLabel);
	isrUttSpinBox = new QSpinBox();
	isrUttSpinBox->setRange(1,75);
	isrUttSpinBox->setValue(35);
	eqHBoxLayout->addWidget(isrUttSpinBox);
	sendToIsrButton = new QPushButton ("Send values to ISR");
	eqHBoxLayout->addWidget(sendToIsrButton);
	eqVBoxLayout->addLayout(eqHBoxLayout);
	wellLabel = new QLabel("Well, all I can hear is ....: ");
	eqVBoxLayout->addWidget(wellLabel);
	isrResultsTextEdit = new QTextEdit();
	eqVBoxLayout->addWidget(isrResultsTextEdit);
	eqHBoxLayout2 = new QHBoxLayout();
	connectToIsrButton = new QPushButton("Connect");
	disconnectFromIsrButton = new QPushButton ("Disconnect");
	enableOutputIsrButton = new QPushButton ("Enable Output");
	disableOutputIsrButton = new QPushButton ("Disable Output");
	resumeLsearchIsrButton = new QPushButton ("Resume LSearch");
	pauseLsearchIsrButton = new QPushButton ("Pause LSearch");
	eqHBoxLayout2->addWidget(connectToIsrButton);
	eqHBoxLayout2->addWidget(disconnectFromIsrButton);
	eqHBoxLayout2->addWidget(enableOutputIsrButton);
	eqHBoxLayout2->addWidget(disableOutputIsrButton);
	eqHBoxLayout2->addWidget(resumeLsearchIsrButton);
	eqHBoxLayout2->addWidget(pauseLsearchIsrButton);
	eqVBoxLayout->addLayout(eqHBoxLayout2);
	quitButton = new QPushButton ("Quit");
	eqVBoxLayout->addWidget(quitButton);
	boost::this_thread::sleep(boost::posix_time::milliseconds(100));
	if (!xmem->isConnected())
	{
		enableConnectButton();
		std::cerr << "test: not connected!" << std::endl;
	}
	else
	{
		disableConnectButton();
		std::cerr << "test: connected!" << std::endl;
	}
	QObject::connect(xmem,SIGNAL ( updated_dB(float, int, int, QString) ), this, SLOT (update_eq (float, int, int, QString) ));
	QObject::connect(xmem,SIGNAL ( newIsrHyp (QString) ), this, SLOT (update_text (QString) ));
	QObject::connect(quitButton, SIGNAL ( clicked()), this, SLOT ( quit()));
	QObject::connect(sendToIsrButton, SIGNAL (clicked()), this, SLOT ( sendToIsr()));
	QObject::connect(xmem, SIGNAL ( disconnected()), this, SLOT (gotDisconnect()));
	QObject::connect(connectToIsrButton, SIGNAL (clicked()), this, SLOT (triggerConnectToIsr()));
	QObject::connect(disconnectFromIsrButton, SIGNAL (clicked()), this, SLOT (triggerDisconnectFromIsr()));
	QObject::connect(enableOutputIsrButton, SIGNAL (clicked()), this, SLOT (triggerEnableOutput()));
	QObject::connect(disableOutputIsrButton, SIGNAL (clicked()), this, SLOT (triggerDisableOutput()));
	QObject::connect(resumeLsearchIsrButton, SIGNAL (clicked()), this, SLOT (triggerResumeLsearch()));
	QObject::connect(pauseLsearchIsrButton, SIGNAL (clicked()), this, SLOT (triggerPauseLsearch()));
}

measureDbMainWindow::~measureDbMainWindow()
{
	QObject::disconnect(connectToIsrButton, SIGNAL (clicked()), this, SLOT (triggerConnectToIsr()));
	QObject::disconnect(disconnectFromIsrButton, SIGNAL (clicked()), this, SLOT (triggerDisconnectFromIsr()));
	QObject::disconnect(enableOutputIsrButton, SIGNAL (clicked()), this, SLOT (triggerEnableOutput()));
	QObject::disconnect(disableOutputIsrButton, SIGNAL (clicked()), this, SLOT (triggerDisableOutput()));
	QObject::disconnect(resumeLsearchIsrButton, SIGNAL (clicked()), this, SLOT (triggerResumeLsearch()));
	QObject::disconnect(pauseLsearchIsrButton, SIGNAL (clicked()), this, SLOT (triggerPauseLsearch()));
	QObject::disconnect(xmem, SIGNAL ( disconnected()), this, SLOT (gotDisconnect()));
	QObject::disconnect(xmem,SIGNAL ( newIsrHyp (QString) ), this, SLOT (update_text (QString) ));
	QObject::disconnect(quitButton, SIGNAL ( clicked()), this, SLOT ( quit()));
	QObject::disconnect(xmem, SIGNAL( updated_dB(float, int, int, QString) ), this, SLOT (update_eq (float, int, int, QString) ));
	QObject::disconnect(sendToIsrButton, SIGNAL (clicked()), this, SLOT ( sendToIsr()));
	delete isrResultsTextEdit;
	delete ew;
	delete eqVBoxLayout;
	delete baseFrame;
}


void
measureDbMainWindow::update_eq(float val, int db_start, int db_utt, QString phase)
{
	ew->setLevel(val);
	if (phase.compare("sleeping") == 0)
		ew->setColor(Qt::green);
	if ((phase.compare("rising") == 0) || (phase.compare("list") == 0))
		ew->setColor(Qt::blue);
	if (phase.compare("falling") == 0)
		ew->setColor(Qt::yellow);
	std::stringstream s;
	QString qs;
	if (val > maxDb)
		maxDb = val;
	averageDb = 0.95*averageDb  + 0.05*val;
	stAverageDb = 0.75*stAverageDb + 0.25*val;
	s << "Level:  " << std::setw(6) << std::setprecision(4) << std::showpoint << val << " db, LTAv.: " << averageDb << " db, STAv.: " << stAverageDb <<" db, Max:  " << maxDb << " db Start: " << db_start << " db Utt: " << db_utt << "db  Phase: " << phase.toStdString();
	dbLabel->setText(s.str().c_str());
}

void
measureDbMainWindow::update_text(QString update)
{
	/*static long long last_timestamp = 0;
	if (timestamp == last_timestamp)
	{
		std::cerr << "Updating hypothesis: ";
		isrResultsTextEdit->moveCursor( QTextCursor::End, QTextCursor::MoveAnchor );
		isrResultsTextEdit->moveCursor( QTextCursor::StartOfLine, QTextCursor::MoveAnchor );
		isrResultsTextEdit->moveCursor( QTextCursor::End, QTextCursor::KeepAnchor );
		std::cerr << isrResultsTextEdit->textCursor().selectedText().toStdString();
		std::cerr << " to " << update.toStdString() << std::endl;
		isrResultsTextEdit->textCursor().removeSelectedText();
	}
	last_timestamp = timestamp;*/
	isrResultsTextEdit->append(update);
}



void
measureDbMainWindow::triggerConnectToIsr()
{
	if (xmem->connectToIsr())
		disableConnectButton();
	else
	{
		QMessageBox* m = new QMessageBox("XCF CONNECT FAILED","Could not subscribe to isr, is dispatcher and isr publisher running?!",QMessageBox::Critical,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		m->exec();
		return;
	}
}

void
measureDbMainWindow::triggerDisconnectFromIsr()
{
	xmem->disconnectFromIsr();
	enableConnectButton();
}



void
measureDbMainWindow::disableConnectButton()
{
	connectToIsrButton->setDisabled(true);
	disconnectFromIsrButton->setDisabled(false);
}

void
measureDbMainWindow::enableConnectButton()
{
	connectToIsrButton->setDisabled(false);
	disconnectFromIsrButton->setDisabled(true);
}

void
measureDbMainWindow::triggerDisableOutput()
{
	XCF::RemoteServerPtr isrRemoteServer;
	try {
	isrRemoteServer = XCF::RemoteServer::create("isr");
	}
	catch (XCF::ServerNotFoundException &ex)
	{
		QMessageBox* m = new QMessageBox("XCF CONNECT FAILED","No server with name isr registered!",QMessageBox::Critical,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		m->exec();
		return;
	}
	catch (XCF::InitializeException &ex)
	{
		QMessageBox* m = new QMessageBox("XCF CONNECT FAILED","Could not connect, is dispatcher running?!",QMessageBox::Critical,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		m->exec();
		return;
	}
	std::string message, result;
	try {
	isrRemoteServer->callMethod("disable_output",message,result);
	}
	catch (XCF::MethodNotFoundException &ex)
	{
		QMessageBox* m = new QMessageBox("XCF SEND FAILED","Remote Server does not know 'disable_output'!",QMessageBox::Critical,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		m->exec();
		return;
	}
}

void
measureDbMainWindow::triggerEnableOutput()
{
	XCF::RemoteServerPtr isrRemoteServer;
	try {
	isrRemoteServer = XCF::RemoteServer::create("isr");
	}
	catch (XCF::ServerNotFoundException &ex)
	{
		QMessageBox* m = new QMessageBox("XCF CONNECT FAILED","No server with name isr registered!",QMessageBox::Critical,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		m->exec();
		return;
	}
	catch (XCF::InitializeException &ex)
	{
		QMessageBox* m = new QMessageBox("XCF CONNECT FAILED","Could not connect, is dispatcher running?!",QMessageBox::Critical,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		m->exec();
		return;
	}
	std::string message, result;
	try {
	isrRemoteServer->callMethod("enable_output",message,result);
	}
	catch (XCF::MethodNotFoundException &ex)
	{
		QMessageBox* m = new QMessageBox("XCF SEND FAILED","Remote Server does not know 'enable_output'!",QMessageBox::Critical,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		m->exec();
		return;
	}
}

void
measureDbMainWindow::triggerPauseLsearch()
{
	XCF::RemoteServerPtr isrRemoteServer;
	try {
	isrRemoteServer = XCF::RemoteServer::create("isr");
	}
	catch (XCF::ServerNotFoundException &ex)
	{
		QMessageBox* m = new QMessageBox("XCF CONNECT FAILED","No server with name isr registered!",QMessageBox::Critical,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		m->exec();
		return;
	}
	catch (XCF::InitializeException &ex)
	{
		QMessageBox* m = new QMessageBox("XCF CONNECT FAILED","Could not connect, is dispatcher running?!",QMessageBox::Critical,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		m->exec();
		return;
	}
	std::string message, result;
	try {
	isrRemoteServer->callMethod("pause_lsearch",message,result);
	}
	catch (XCF::MethodNotFoundException &ex)
	{
		QMessageBox* m = new QMessageBox("XCF SEND FAILED","Remote Server does not know 'pause_lsearch'!",QMessageBox::Critical,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		m->exec();
		return;
	}
}

void
measureDbMainWindow::triggerResumeLsearch()
{
	XCF::RemoteServerPtr isrRemoteServer;
	try {
	isrRemoteServer = XCF::RemoteServer::create("isr");
	}
	catch (XCF::ServerNotFoundException &ex)
	{
		QMessageBox* m = new QMessageBox("XCF CONNECT FAILED","No server with name isr registered!",QMessageBox::Critical,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		m->exec();
		return;
	}
	catch (XCF::InitializeException &ex)
	{
		QMessageBox* m = new QMessageBox("XCF CONNECT FAILED","Could not connect, is dispatcher running?!",QMessageBox::Critical,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		m->exec();
		return;
	}
	std::string message, result;
	try {
	isrRemoteServer->callMethod("resume_lsearch",message,result);
	}
	catch (XCF::MethodNotFoundException &ex)
	{
		QMessageBox* m = new QMessageBox("XCF SEND FAILED","Remote Server does not know 'resume_lsearch'!",QMessageBox::Critical,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		m->exec();
		return;
	}
}

void
measureDbMainWindow::gotDisconnect()
{
	disableConnectButton();
	QMessageBox* m = new QMessageBox("XCF DISCONNECT","A disconnect from the ISR publisher occured!",QMessageBox::Critical,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
	m->exec();
}

void
measureDbMainWindow::quit()
{
	emit gotQuit();
}

void
measureDbMainWindow::sendToIsr()
{
	int startDb = isrStartSpinBox->value();
	int uttDb = isrUttSpinBox->value();
	XCF::RemoteServerPtr isrRemoteServer;
	try {
	isrRemoteServer = XCF::RemoteServer::create("isr");
	}
	catch (XCF::ServerNotFoundException &ex)
	{
		QMessageBox* m = new QMessageBox("XCF CONNECT FAILED","No server with name isr registered!",QMessageBox::Critical,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		m->exec();
		std::cerr << "ServerNotFoundException" << std::endl;
		return;
	}
	catch (XCF::InitializeException &ex)
	{
		QMessageBox* m = new QMessageBox("XCF CONNECT FAILED","Could not connect, is dispatcher running?!",QMessageBox::Critical,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		m->exec();
		std::cerr << "Catching InitializeException" << std::endl;
		return;
	}
	xmltio::Location loc ("<sr></sr>","/sr");
	loc["db_start"] = startDb;
	loc["db_utt"] = uttDb;
	std::string result;
	try {
	isrRemoteServer->callMethod("set_sr_params",loc.getDocumentText(),result);
	}
	catch (XCF::MethodNotFoundException &ex)
	{
		QMessageBox* m = new QMessageBox("XCF SEND FAILED","Remote Server does not know 'resume_lsearch'!",QMessageBox::Critical,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		m->exec();
		std::cerr << "Catching MethodNotFoundException " << std::endl;
	}
}

int
main (int argc, char** argv)
{
	QApplication myapp (argc,argv);
	measureDbMainWindow w;
	w.setGeometry(200,100,1000,650);
	w.show();
	QObject::connect(&w, SIGNAL ( gotQuit() ), &myapp, SLOT ( quit() ));

	return myapp.exec();
}


