/*
 * xcfMemoryListener.cpp
 *
 *  Created on: Jun 11, 2012
 *      Author: mschoepf
 */

#include "xcfMemoryListener.h"

bool
xcfMemoryListener::connectToIsr()
{
	try {
		_subscriber = XCF::Subscriber::create("isr");
		} catch (XCF::NameNotFoundException &e)
		{
			std::cerr << "Could not find publisher with name isr" << std::endl;
			connected = false;
			return false;
		}
		catch (XCF::InitializeException &ex)
		{
			std::cerr << "Could not find dispatcher" << std::endl;
			connected = false;
			return false;
		}
	connected = true;
	return true;
}

void
xcfMemoryListener::disconnectFromIsr()
{
	connected = false;
	boost::this_thread::sleep(boost::posix_time::milliseconds(200));
	_subscriber->destroy();
}

xcfMemoryListener::xcfMemoryListener() {
	connected = false;
	readThreadPtr.reset(new boost::thread(&xcfMemoryListener::runner,this));
}

xcfMemoryListener::~xcfMemoryListener() {
	readThreadPtr->interrupt();
	readThreadPtr->join();
	//_subscriber->destroy();
}

bool
xcfMemoryListener::isConnected() const
{
	return connected;
}

void
xcfMemoryListener::runner()
{
	float db;
	int isr_signal_db_utt;
	int isr_signal_db_start;
	std::string data;
	long long timestamp;
	if (connectToIsr())
		std::cerr << "connected!!" << std::endl;
	for(;;)
	{
		try {
			boost::this_thread::interruption_point();
			if (!connected)
			{
				boost::this_thread::sleep(boost::posix_time::milliseconds(500));
				continue;
			}
		}
		catch (boost::thread_interrupted &ti)
		{
			break;
		}
		try
		{
			data = _subscriber->receive();
		}
		catch (XCF::CommunicationException &ex)
		{
			connected = false;
			if (!_subscriber->isAlive())
			{
				connected = false;
				emit disconnected();
			}
		}
		try {
			xmltio::Location loc (data,xmltio::XPath("isr_status/@db"));
			db = xmltio::extract<float>(loc);
			isr_signal_db_start = xmltio::extract<int>(xmltio::Location(data,xmltio::XPath("isr_status/@isr_signal_db_start")));
			isr_signal_db_utt = xmltio::extract<int>(xmltio::Location(data,xmltio::XPath("isr_status/@isr_signal_db_utt")));
			std::string phase = xmltio::extract<std::string>(xmltio::Location(data,xmltio::XPath("isr_status/@phase")));
			QString sendme = phase.c_str();
			emit updated_dB (db,isr_signal_db_start, isr_signal_db_utt,sendme);
		} catch (xmltio::NotFoundException &ex)
		{
		}
		try {
			xmltio::Location loc (data,xmltio::XPath("speech_hyp"));
			timestamp = xmltio::extract<long long>(loc[xmltio::XPath("TIMESTAMP")]);
			std::stringstream s;
			int stable = xmltio::extract<int>(loc[xmltio::XPath("@stable")]);
			s << timestamp;
			if (stable == 0)
				s << " (unstable): ";
			else
				s << ": ";
			for(xmltio::XPathIterator i = xmltio::XPath("seq/part").evaluate(loc); i; ++i) {
				xmltio::Location l(*i);
				std::string temp = xmltio::extract<std::string>(l);
				s << temp << " ";
				std::cerr << s << std::endl;
			}
			QString isrResult = s.str().c_str();
			emit newIsrHyp (isrResult);
		} catch (xmltio::NotFoundException &ex)
		{
		}
	}
}

